@@ -1,6 +1,6 @@
 /*****************************************************************************/
 /*  chmod_dialog.c - the chmod dialog box                                    */
-/*  Copyright (C) 1998-2002 Brian Masney <masneyb@seul.org>                  */
+/*  Copyright (C) 1998-2003 Brian Masney <masneyb@gftp.org>                  */
 /*                                                                           */
 /*  This program is free software; you can redistribute it and/or modify     */
 /*  it under the terms of the GNU General Public License as published by     */
@@ -18,17 +18,106 @@
 /*****************************************************************************/
 
 #include "gftp-gtk.h"
-
-static void dochmod 				( GtkWidget * widget, 
-						  gftp_window_data * wdata );
-static void *do_chmod_thread 			( void * data );
-static RETSIGTYPE sig_chmodquit                 ( int signo );
-
+static const char cvsid[] = "$Id: chmod_dialog.c,v 1.20 2004/12/24 11:26:52 masneyb Exp $";
 
 static GtkWidget *suid, *sgid, *sticky, *ur, *uw, *ux, *gr, *gw, *gx, *or, *ow,
                  *ox;
-static sigjmp_buf chmodenvir;
-static int mode; 
+static mode_t mode; 
+
+
+static int
+do_chmod_thread (gftpui_callback_data * cdata)
+{
+  GList * filelist, * templist;
+  gftp_window_data * wdata;
+  gftp_file * tempfle;
+  int error, num;
+
+  wdata = cdata->uidata;
+  error = 0;
+
+  filelist = wdata->files;
+  templist = gftp_gtk_get_list_selection (wdata);
+  num = 0;
+  while (templist != NULL)
+    {
+      templist = get_next_selection (templist, &filelist, &num);
+      tempfle = filelist->data;
+
+      if (gftp_chmod (wdata->request, tempfle->file, mode) != 0)
+        error = 1;
+
+      if (!GFTP_IS_CONNECTED (wdata->request))
+        break;
+    }
+
+  return (error);
+}
+
+
+static void
+dochmod (GtkWidget * widget, gftp_window_data * wdata)
+{
+  gftpui_callback_data * cdata;
+
+  mode = 0;
+  if (GTK_TOGGLE_BUTTON (suid)->active)
+    mode |= S_ISUID;
+  if (GTK_TOGGLE_BUTTON (sgid)->active)
+    mode |= S_ISGID;
+  if (GTK_TOGGLE_BUTTON (sticky)->active)
+    mode |= S_ISVTX;
+
+  if (GTK_TOGGLE_BUTTON (ur)->active)
+    mode |= S_IRUSR;
+  if (GTK_TOGGLE_BUTTON (uw)->active)
+    mode |= S_IWUSR;
+  if (GTK_TOGGLE_BUTTON (ux)->active)
+    mode |= S_IXUSR;
+
+  if (GTK_TOGGLE_BUTTON (gr)->active)
+    mode |= S_IRGRP;
+  if (GTK_TOGGLE_BUTTON (gw)->active)
+    mode |= S_IWGRP;
+  if (GTK_TOGGLE_BUTTON (gx)->active)
+    mode |= S_IXGRP;
+
+  if (GTK_TOGGLE_BUTTON (or)->active)
+    mode |= S_IROTH;
+  if (GTK_TOGGLE_BUTTON (ow)->active)
+    mode |= S_IWOTH;
+  if (GTK_TOGGLE_BUTTON (ox)->active)
+    mode |= S_IXOTH;
+
+  if (check_reconnect (wdata) < 0)
+    return;
+
+  cdata = g_malloc0 (sizeof (*cdata));
+  cdata->request = wdata->request;
+  cdata->uidata = wdata;
+  cdata->run_function = do_chmod_thread;
+
+  gftpui_common_run_callback_function (cdata);
+
+  g_free (cdata);
+}
+
+
+#if GTK_MAJOR_VERSION > 1
+static void
+chmod_action (GtkWidget * widget, gint response, gpointer wdata)
+{
+  switch (response)
+    {
+      case GTK_RESPONSE_OK:
+        dochmod (widget, wdata);
+        /* no break */
+      default:
+        gtk_widget_destroy (widget);
+    }
+}
+#endif
+
 
 void
 chmod_dialog (gpointer data)
@@ -40,22 +129,35 @@ chmod_dialog (gpointer data)
   int num;
 
   wdata = data;
-  if (!check_status (_("Chmod"), wdata, wdata->request->use_threads, 1, 1, 
-                     wdata->request->chmod != NULL))
+  if (!check_status (_("Chmod"), wdata, gftpui_common_use_threads (wdata->request), 0, 1, wdata->request->chmod != NULL))
     return;
 
+#if GTK_MAJOR_VERSION == 1
   dialog = gtk_dialog_new ();
   gtk_window_set_title (GTK_WINDOW (dialog), _("Chmod"));
-  gtk_window_set_wmclass (GTK_WINDOW(dialog), "Chmod", "gFTP");
-  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
   gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->action_area),
                               5);
-  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
   gtk_box_set_homogeneous (GTK_BOX (GTK_DIALOG (dialog)->action_area), TRUE);
+#else
+  dialog = gtk_dialog_new_with_buttons (_("Chmod"), NULL, 0,
+                                        GTK_STOCK_OK,
+                                        GTK_RESPONSE_OK,
+                                        GTK_STOCK_CANCEL,
+                                        GTK_RESPONSE_CANCEL,
+                                        NULL);
+#endif
+  gtk_window_set_wmclass (GTK_WINDOW(dialog), "Chmod", "gFTP");
   gtk_window_set_position (GTK_WINDOW (dialog), GTK_WIN_POS_MOUSE);
-  gtk_signal_connect_object (GTK_OBJECT (dialog), "delete_event",
-			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
-			     GTK_OBJECT (dialog));
+  gtk_box_set_spacing (GTK_BOX (GTK_DIALOG (dialog)->vbox), 5);
+  gtk_container_border_width (GTK_CONTAINER (GTK_DIALOG (dialog)->vbox), 10);
+  gtk_widget_realize (dialog);
+
+  if (gftp_icon != NULL)
+    {
+      gdk_window_set_icon (dialog->window, NULL, gftp_icon->pixmap,
+                           gftp_icon->bitmap);
+      gdk_window_set_icon_name (dialog->window, gftp_version);
+    }
 
   tempwid = gtk_label_new (_("You can now adjust the attributes of your file(s)\nNote: Not all ftp servers support the chmod feature"));
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->vbox), tempwid, FALSE,
@@ -147,7 +249,8 @@ chmod_dialog (gpointer data)
   gtk_box_pack_start (GTK_BOX (vbox), ox, FALSE, FALSE, 0);
   gtk_widget_show (ox);
 
-  tempwid = gtk_button_new_with_label (_("Change"));
+#if GTK_MAJOR_VERSION == 1
+  tempwid = gtk_button_new_with_label (_("OK"));
   GTK_WIDGET_SET_FLAGS (tempwid, GTK_CAN_DEFAULT);
   gtk_box_pack_start (GTK_BOX (GTK_DIALOG (dialog)->action_area), tempwid,
 		      TRUE, TRUE, 0);
@@ -167,157 +270,46 @@ chmod_dialog (gpointer data)
 			     GTK_SIGNAL_FUNC (gtk_widget_destroy),
 			     GTK_OBJECT (dialog));
   gtk_widget_show (tempwid);
+#else
+  g_signal_connect (GTK_OBJECT (dialog), "response",
+                    G_CALLBACK (chmod_action), wdata);
+#endif
 
   if (IS_ONE_SELECTED (wdata))
     {
       filelist = wdata->files;
-      templist = GTK_CLIST (wdata->listbox)->selection;
+      templist = gftp_gtk_get_list_selection (wdata);
       num = 0;
       templist = get_next_selection (templist, &filelist, &num);
       tempfle = filelist->data;
+
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (suid),
-				    tolower (tempfle->attribs[3]) == 's');
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sgid),
-				    tolower (tempfle->attribs[6]) == 's');
-      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sticky),
-				    tolower (tempfle->attribs[9]) == 't');
+                                    tempfle->st_mode & S_ISUID);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ur),
-				    tempfle->attribs[1] == 'r');
+                                    tempfle->st_mode & S_IRUSR);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (uw),
-				    tempfle->attribs[2] == 'w');
+                                    tempfle->st_mode & S_IWUSR);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ux),
-				    tempfle->attribs[3] == 'x' ||
-                                    tempfle->attribs[3] == 's');
+                                    tempfle->st_mode & S_IXUSR);
+
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sgid),
+                                    tempfle->st_mode & S_ISGID);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gr),
-				    tempfle->attribs[4] == 'r');
+                                    tempfle->st_mode & S_IRGRP);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gw),
-				    tempfle->attribs[5] == 'w');
+                                    tempfle->st_mode & S_IWGRP);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (gx),
-				    tempfle->attribs[6] == 'x' ||
-                                    tempfle->attribs[6] == 's');
+                                    tempfle->st_mode & S_IXGRP);
+
+      gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (sticky),
+                                    tempfle->st_mode & S_ISVTX);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (or),
-				    tempfle->attribs[7] == 'r');
+                                    tempfle->st_mode & S_IROTH);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ow),
-				    tempfle->attribs[8] == 'w');
+                                    tempfle->st_mode & S_IWOTH);
       gtk_toggle_button_set_active (GTK_TOGGLE_BUTTON (ox),
-				    tempfle->attribs[9] == 'x' ||
-                                    tempfle->attribs[9] == 't');
+                                    tempfle->st_mode & S_IXOTH);
     }
   gtk_widget_show (dialog);
 }
 
-
-static void
-dochmod (GtkWidget * widget, gftp_window_data * wdata)
-{
-  int cur;
-
-  mode = 0;
-  if (GTK_TOGGLE_BUTTON (suid)->active)
-    mode += 4;
-  if (GTK_TOGGLE_BUTTON (sgid)->active)
-    mode += 2;
-  if (GTK_TOGGLE_BUTTON (sticky)->active)
-    mode += 1;
-
-  cur = 0;
-  if (GTK_TOGGLE_BUTTON (ur)->active)
-    cur += 4;
-  if (GTK_TOGGLE_BUTTON (uw)->active)
-    cur += 2;
-  if (GTK_TOGGLE_BUTTON (ux)->active)
-    cur += 1;
-  mode = mode * 10 + cur;
-
-  cur = 0;
-  if (GTK_TOGGLE_BUTTON (gr)->active)
-    cur += 4;
-  if (GTK_TOGGLE_BUTTON (gw)->active)
-    cur += 2;
-  if (GTK_TOGGLE_BUTTON (gx)->active)
-    cur += 1;
-  mode = mode * 10 + cur;
-
-  cur = 0;
-  if (GTK_TOGGLE_BUTTON (or)->active)
-    cur += 4;
-  if (GTK_TOGGLE_BUTTON (ow)->active)
-    cur += 2;
-  if (GTK_TOGGLE_BUTTON (ox)->active)
-    cur += 1;
-  mode = mode * 10 + cur;
-
-  if (check_reconnect (wdata) < 0)
-    return;
-
-   if ((int) generic_thread (do_chmod_thread, wdata))
-    refresh (wdata);
-}
-
-
-static void *
-do_chmod_thread (void * data)
-{
-  GList * filelist, * templist;
-  gftp_window_data * wdata;
-  int success, num, sj;
-  gftp_file * tempfle;
-
-  wdata = data;
-  wdata->request->user_data = (void *) 0x01;
-
-  if (wdata->request->use_threads)
-    {
-      sj = sigsetjmp (chmodenvir, 1);
-      signal (SIGINT, sig_chmodquit);
-      signal (SIGALRM, sig_chmodquit);
-    }
-  else
-    sj = 0;
-
-  success = 0;
-  if (sj == 0)
-    { 
-      filelist = wdata->files;
-      templist = GTK_CLIST (wdata->listbox)->selection;
-      num = 0;
-      while (templist != NULL)
-        {
-          templist = get_next_selection (templist, &filelist, &num);
-          tempfle = filelist->data;
-          if (wdata->request->connect_timeout > 0)
-            alarm (wdata->request->connect_timeout);
-          if (gftp_chmod (wdata->request, tempfle->file, mode) == 0)
-            success = 1;
-          if (!GFTP_IS_CONNECTED (wdata->request))
-            break;
-        }
-      alarm (0);
-    }
-  else
-    {
-      gftp_disconnect (wdata->request);
-      wdata->request->logging_function (gftp_logging_error, 
-                                        wdata->request->user_data,
-                                        _("Operation canceled\n"));
-    }
-
-  if (wdata->request->use_threads)
-    {
-      signal (SIGINT, SIG_DFL);
-      signal (SIGALRM, SIG_IGN);
-    }
-
-  wdata->request->user_data = NULL;
-  wdata->request->stopable = 0;
-  return ((void *) success);
-}
-
-
-static RETSIGTYPE
-sig_chmodquit (int signo)
-{
-  signal (signo, sig_chmodquit);
-  siglongjmp (chmodenvir, signo == SIGINT ? 1 : 2);
-}
-
